package com.faraphel.tasks_valider

import android.content.Context
import android.content.pm.PackageManager
import android.Manifest
import android.annotation.SuppressLint
import android.net.wifi.p2p.*
import android.os.Build
import android.os.Bundle
import android.util.Log
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.annotation.RequiresApi
import com.faraphel.tasks_valider.connectivity.wifi_p2p.WifiP2pHelper
import com.faraphel.tasks_valider.database.Database
import com.faraphel.tasks_valider.ui.screen.RoomScreen


class MainActivity : ComponentActivity() {
    private var PERMISSION_ACCESS_FINE_LOCATION = 1001  ///< permission code for the fi
    private var p2pHelper: WifiP2pHelper? = null  ///< the Wi-Fi Direct helper

    companion object {
        private lateinit var database: Database  ///< the database manager
    }

    @RequiresApi(Build.VERSION_CODES.O)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        // ----- CONNECTION

        // TOOD: more check on permissions
        if (!this.packageManager.hasSystemFeature(PackageManager.FEATURE_WIFI_DIRECT)) {
            Log.e("wifi-p2p", "this device does not support the WiFi-Direct feature")
            return
        }

        if (this.checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            // TODO(Faraphel): should be used with shouldShowRequestPermissionRationale, with a check
            this.requestPermissions(arrayOf(Manifest.permission.ACCESS_FINE_LOCATION), PERMISSION_ACCESS_FINE_LOCATION)
        }

        // get the WiFi-Direct manager
        val p2pManager: WifiP2pManager? = this.getSystemService(Context.WIFI_P2P_SERVICE) as WifiP2pManager?
        if (p2pManager == null) {
            Log.e("wifi-p2p", "cannot access the WiFi-Direct manager")
            return
        }

        // get the Wifi-Direct channel
        val p2pChannel: WifiP2pManager.Channel = p2pManager.initialize(this, this.mainLooper, null)

        // create a helper for handling the WiFi-Direct
        this.p2pHelper = WifiP2pHelper(p2pManager, p2pChannel)

        // open the room selection screen
        this.setContent {
            RoomScreen(this.p2pHelper!!)
        }
    }

    @RequiresApi(Build.VERSION_CODES.O)
    @SuppressLint("UnspecifiedRegisterReceiverFlag")
    override fun onResume() {
        super.onResume()

        // enable the WiFi-Direct events
        this.registerReceiver(this.p2pHelper, WifiP2pHelper.ALL_INTENT_FILTER)
    }

    override fun onPause() {
        super.onPause()

        // disable the WiFi-Direct events
        this.unregisterReceiver(this.p2pHelper)
    }
}
